//
//  VCLLoggingViewController.swift
//
//  Created by CS193p Instructor.
//  Copyright © 2015-17 Stanford University. All rights reserved.
//
//  Modified by Steve for CS 326.
//

import UIKit

class LogLifeCycleViewController : UIViewController
{
  private struct LogGlobals {
    var instanceCounts = [String:Int]()
    var lastLogTime = Date()
    var indentationInterval: TimeInterval = 1
  }
  
  private static var logGlobals = LogGlobals()
  
  private static func logPrefix(for className: String) -> String {
    if logGlobals.lastLogTime.timeIntervalSinceNow < -logGlobals.indentationInterval {
      print("")
    }
    logGlobals.lastLogTime = Date()
    return className
  }
  
  private static func bumpInstanceCount(for className: String) -> Int {
    logGlobals.instanceCounts[className] = (logGlobals.instanceCounts[className] ?? 0) + 1
    return logGlobals.instanceCounts[className]!
  }
  
  private var instanceCount: Int!
  
  private func logLifeCycleEvent(_ msg: String) {
    let className = String(describing: type(of: self))
    if instanceCount == nil {
      instanceCount =  LogLifeCycleViewController.bumpInstanceCount(for: className)
    }
    print("\(LogLifeCycleViewController.logPrefix(for: className))(\(instanceCount!)) \(msg)")
  }
  
  required init?(coder aDecoder: NSCoder) {
    super.init(coder: aDecoder)
    logLifeCycleEvent("init(coder:) - created via InterfaceBuilder ")
  }
  
  override init(nibName nibNameOrNil: String?, bundle nibBundleOrNil: Bundle?) {
    super.init(nibName: nibNameOrNil, bundle: nibBundleOrNil)
    logLifeCycleEvent("init(nibName:bundle:) - create in code")
  }
  
  deinit {
    logLifeCycleEvent("left the heap")
  }
  
  override func awakeFromNib() {
    logLifeCycleEvent("awakeFromNib()")
  }
  
  override func viewDidLoad() {
    super.viewDidLoad()
    logLifeCycleEvent("viewDidLoad()")
  }
  
  override func viewWillAppear(_ animated: Bool) {
    super.viewWillAppear(animated)
    logLifeCycleEvent("viewWillAppear(animated = \(animated))")
  }
  
  override func viewDidAppear(_ animated: Bool) {
    super.viewDidAppear(animated)
    logLifeCycleEvent("viewDidAppear(animated = \(animated))")
  }
  
  override func viewWillDisappear(_ animated: Bool) {
    super.viewWillDisappear(animated)
    logLifeCycleEvent("viewWillDisappear(animated = \(animated))")
  }
  
  override func viewDidDisappear(_ animated: Bool) {
    super.viewDidDisappear(animated)
    logLifeCycleEvent("viewDidDisappear(animated = \(animated))")
  }
  
  override func didReceiveMemoryWarning() {
    super.didReceiveMemoryWarning()
    logLifeCycleEvent("didReceiveMemoryWarning()")
  }
  
  override func viewWillLayoutSubviews() {
    super.viewWillLayoutSubviews()
    logLifeCycleEvent("viewWillLayoutSubviews() bounds.size = \(view.bounds.size)")
  }
  
  override func viewDidLayoutSubviews() {
    super.viewDidLayoutSubviews()
    logLifeCycleEvent("viewDidLayoutSubviews() bounds.size = \(view.bounds.size)")
  }
  
  override func viewWillTransition(to size: CGSize, with coordinator: UIViewControllerTransitionCoordinator) {
    super.viewWillTransition(to: size, with: coordinator)
    logLifeCycleEvent("viewWillTransition(to: \(size), with: coordinator)")
    coordinator.animate(alongsideTransition: { (context: UIViewControllerTransitionCoordinatorContext!) -> Void in
      self.logLifeCycleEvent("begin animate(alongsideTransition:completion:)")
    }, completion: { context -> Void in
      self.logLifeCycleEvent("end animate(alongsideTransition:completion:)")
    })
  }
}
